@@ -23,13 +23,11 @@ static void print_usage(FILE *f)
 {
 	fprintf(f,
 		"Usage: ip link set DEVICE type can\n"
-		"\t[ bitrate BITRATE [ sample-point SAMPLE-POINT] ] | \n"
-		"\t[ tq TQ prop-seg PROP_SEG phase-seg1 PHASE-SEG1\n "
-		"\t  phase-seg2 PHASE-SEG2 [ sjw SJW ] ]\n"
+		"\t[ bitrate BITRATE [ sample-point SAMPLE-POINT] ] |\n"
+		"\t[ tq TQ prop-seg PROP_SEG phase-seg1 PHASE-SEG1\n \t  phase-seg2 PHASE-SEG2 [ sjw SJW ] ]\n"
 		"\n"
-		"\t[ dbitrate BITRATE [ dsample-point SAMPLE-POINT] ] | \n"
-		"\t[ dtq TQ dprop-seg PROP_SEG dphase-seg1 PHASE-SEG1\n "
-		"\t  dphase-seg2 PHASE-SEG2 [ dsjw SJW ] ]\n"
+		"\t[ dbitrate BITRATE [ dsample-point SAMPLE-POINT] ] |\n"
+		"\t[ dtq TQ dprop-seg PROP_SEG dphase-seg1 PHASE-SEG1\n \t  dphase-seg2 PHASE-SEG2 [ dsjw SJW ] ]\n"
 		"\n"
 		"\t[ loopback { on | off } ]\n"
 		"\t[ listen-only { on | off } ]\n"
@@ -37,6 +35,7 @@ static void print_usage(FILE *f)
 		"\t[ one-shot { on | off } ]\n"
 		"\t[ berr-reporting { on | off } ]\n"
 		"\t[ fd { on | off } ]\n"
+		"\t[ fd-non-iso { on | off } ]\n"
 		"\t[ presume-ack { on | off } ]\n"
 		"\n"
 		"\t[ restart-ms TIME-MS ]\n"
@@ -72,7 +71,7 @@ static int get_float(float *val, const char *arg)
 	return 0;
 }
 
-static void set_ctrlmode(char* name, char *arg,
+static void set_ctrlmode(char *name, char *arg,
 			 struct can_ctrlmode *cm, __u32 flags)
 {
 	if (strcmp(arg, "on") == 0) {
@@ -100,6 +99,7 @@ static void print_ctrlmode(FILE *f, __u32 cm)
 	_PF(CAN_CTRLMODE_ONE_SHOT, "ONE-SHOT");
 	_PF(CAN_CTRLMODE_BERR_REPORTING, "BERR-REPORTING");
 	_PF(CAN_CTRLMODE_FD, "FD");
+	_PF(CAN_CTRLMODE_FD_NON_ISO, "FD-NON-ISO");
 	_PF(CAN_CTRLMODE_PRESUME_ACK, "PRESUME-ACK");
 #undef _PF
 	if (cm)
@@ -110,11 +110,9 @@ static void print_ctrlmode(FILE *f, __u32 cm)
 static int can_parse_opt(struct link_util *lu, int argc, char **argv,
 			 struct nlmsghdr *n)
 {
-	struct can_bittiming bt, dbt;
+	struct can_bittiming bt = {}, dbt = {};
 	struct can_ctrlmode cm = {0, 0};
 
-	memset(&bt, 0, sizeof(bt));
-	memset(&dbt, 0, sizeof(dbt));
 	while (argc > 0) {
 		if (matches(*argv, "bitrate") == 0) {
 			NEXT_ARG();
@@ -203,6 +201,10 @@ static int can_parse_opt(struct link_util *lu, int argc, char **argv,
 			NEXT_ARG();
 			set_ctrlmode("fd", *argv, &cm,
 				     CAN_CTRLMODE_FD);
+		} else if (matches(*argv, "fd-non-iso") == 0) {
+			NEXT_ARG();
+			set_ctrlmode("fd-non-iso", *argv, &cm,
+				     CAN_CTRLMODE_FD_NON_ISO);
 		} else if (matches(*argv, "presume-ack") == 0) {
 			NEXT_ARG();
 			set_ctrlmode("presume-ack", *argv, &cm,
@@ -283,11 +285,9 @@ static void can_print_opt(struct link_util *lu, FILE *f, struct rtattr *tb[])
 	if (tb[IFLA_CAN_BITTIMING]) {
 		struct can_bittiming *bt = RTA_DATA(tb[IFLA_CAN_BITTIMING]);
 
-		fprintf(f, "\n	  "
-			"bitrate %d sample-point %.3f ",
+		fprintf(f, "\n	  bitrate %d sample-point %.3f ",
 			bt->bitrate, (float)bt->sample_point / 1000.);
-		fprintf(f, "\n	  "
-			"tq %d prop-seg %d phase-seg1 %d phase-seg2 %d sjw %d",
+		fprintf(f, "\n	  tq %d prop-seg %d phase-seg1 %d phase-seg2 %d sjw %d",
 			bt->tq, bt->prop_seg, bt->phase_seg1, bt->phase_seg2,
 			bt->sjw);
 	}
@@ -296,8 +296,7 @@ static void can_print_opt(struct link_util *lu, FILE *f, struct rtattr *tb[])
 		struct can_bittiming_const *btc =
 			RTA_DATA(tb[IFLA_CAN_BITTIMING_CONST]);
 
-		fprintf(f, "\n	  "
-			"%s: tseg1 %d..%d tseg2 %d..%d "
+		fprintf(f, "\n	  %s: tseg1 %d..%d tseg2 %d..%d "
 			"sjw 1..%d brp %d..%d brp-inc %d",
 			btc->name, btc->tseg1_min, btc->tseg1_max,
 			btc->tseg2_min, btc->tseg2_max, btc->sjw_max,
@@ -308,11 +307,9 @@ static void can_print_opt(struct link_util *lu, FILE *f, struct rtattr *tb[])
 		struct can_bittiming *dbt =
 			RTA_DATA(tb[IFLA_CAN_DATA_BITTIMING]);
 
-		fprintf(f, "\n	  "
-			"dbitrate %d dsample-point %.3f ",
+		fprintf(f, "\n	  dbitrate %d dsample-point %.3f ",
 			dbt->bitrate, (float)dbt->sample_point / 1000.);
-		fprintf(f, "\n	  "
-			"dtq %d dprop-seg %d dphase-seg1 %d "
+		fprintf(f, "\n	  dtq %d dprop-seg %d dphase-seg1 %d "
 			"dphase-seg2 %d dsjw %d",
 			dbt->tq, dbt->prop_seg, dbt->phase_seg1,
 			dbt->phase_seg2, dbt->sjw);
@@ -322,8 +319,7 @@ static void can_print_opt(struct link_util *lu, FILE *f, struct rtattr *tb[])
 		struct can_bittiming_const *dbtc =
 			RTA_DATA(tb[IFLA_CAN_DATA_BITTIMING_CONST]);
 
-		fprintf(f, "\n	  "
-			"%s: dtseg1 %d..%d dtseg2 %d..%d "
+		fprintf(f, "\n	  %s: dtseg1 %d..%d dtseg2 %d..%d "
 			"dsjw 1..%d dbrp %d..%d dbrp-inc %d",
 			dbtc->name, dbtc->tseg1_min, dbtc->tseg1_max,
 			dbtc->tseg2_min, dbtc->tseg2_max, dbtc->sjw_max,
@@ -345,8 +341,7 @@ static void can_print_xstats(struct link_util *lu,
 
 	if (xstats && RTA_PAYLOAD(xstats) == sizeof(*stats)) {
 		stats = RTA_DATA(xstats);
-		fprintf(f, "\n	  "
-			"re-started bus-errors arbit-lost "
+		fprintf(f, "\n	  re-started bus-errors arbit-lost "
 			"error-warn error-pass bus-off");
 		fprintf(f, "\n	  %-10d %-10d %-10d %-10d %-10d %-10d",
 			stats->restarts, stats->bus_error,
